@@ -1,18 +1,21 @@
 =pod #####################################################################################
 
-=head1 Apache::Voodoo::ServerConfig
+=head1 NAME
 
-$Id: ServerConfig.pm 2597 2005-09-15 16:33:41Z medwards $
+Apache::Voodoo::ServerConfig
 
-=head1 Initial Coding: Maverick
+=head1 VERSION
 
-This handles all of the config file parsing and module loading.
+$Id: ServerConfig.pm 4269 2006-11-27 21:14:10Z medwards $
+
+=head1 SYNOPSIS
+
+This modules is used internally by Voodoo for application setup and module loading/reloading.
 
 =cut ################################################################################
-
 package Apache::Voodoo::ServerConfig;
 
-$VERSION = '1.13';
+$VERSION = '1.21';
 
 use strict;
 use Config::General;
@@ -24,9 +27,14 @@ sub new {
 
 	bless $self, $class;
 
-	my $conf = shift;
-	if (defined($conf)) {
-		$self->load($conf);
+	$self->{'id'}        = shift;
+	$self->{'conf_file'} = shift;
+
+	if (defined($self->{'id'}) && defined($self->{'conf_file'})) {
+		$self->load();
+	}
+	else {
+		$self->{'errors'} = "ID and configuration file paths are requried parameters";
 	}
 
 	return $self;
@@ -34,9 +42,9 @@ sub new {
 
 sub load {
 	my $self = shift;
-	my $conf_file = shift;
 
-	$self->load_config($conf_file);
+	# load the configuration file.
+	$self->load_config();
 
 	# get the list of modules we're going to use
 	foreach (keys %{$self->{'modules'}}) {
@@ -55,14 +63,15 @@ sub load {
 
 sub load_config {
 	my $self = shift;
-	my $conf_file = shift;
 
-	my $conf = Config::General->new($conf_file);
+	my $conf = Config::General->new(
+		'-ConfigFile' => $self->{'conf_file'},
+		'-IncludeRelative' => 1,
+		'-UseApacheInclude' => 1
+	);
+
 	my %conf = $conf->getall();
 
-	my ($id) = ($conf_file =~ /([a-zA-Z][\w-]*)\.conf$/);
-	$self->{'id'} = $id;
-
 	$self->{'base_package'} = $conf{'base_package'} || $self->{'id'};
 
 	$self->{'session_dir'}     = $conf{'session_dir'};
@@ -92,11 +101,9 @@ sub load_config {
 	}
 
 	if ($self->{'dynamic_loading'}) {
-		$self->{'conf_mtime'}  = (stat($conf_file))[9];
-		$self->{'conf_file'}   = $conf_file;
+		$self->{'conf_mtime'}  = (stat($self->{'conf_file'}))[9];
 	}
 
-	$self->debug($conf{'database'});
 	if (defined($conf{'database'})) {
 		my $db;
 		if (ref($conf{'database'}) eq "ARRAY") {
@@ -185,7 +192,6 @@ sub map_uri {
 
 sub refresh {
 	my $self = shift;
-	my $page = shift;
 
 	# bypass if we're not doing dynamic loading
 	return unless $self->{'dynamic_loading'};
